/*!
 * \file   include/TFEL/Math/Vector/TVectorFromTinyMatrixColumnView2.hxx
 * \brief This class provides a view of an tvector provided a given
 * runtime offset. This class is used by the behaviours class
 * generated by the MFrontImplicitParser.
 * Tvector From Matric Column
 * View \author Thomas Helfer \date 16 oct 2008
 * \copyright Copyright (C) 2006-2018 CEA/DEN, EDF R&D. All rights 
 * reserved. 
 * This project is publicly released under either the GNU GPL Licence 
 * or the CECILL-A licence. A copy of thoses licences are delivered 
 * with the sources of TFEL. CEA or EDF may also distribute this 
 * project under specific licensing conditions. 
 */

#ifndef LIB_TFEL_MATH_TVECTORFROMTINYMATRIXCOLUMNVIEW2_HXX
#define LIB_TFEL_MATH_TVECTORFROMTINYMATRIXCOLUMNVIEW2_HXX 

#include"TFEL/Metaprogramming/StaticAssert.hxx"
#include"TFEL/Math/General/EmptyRunTimeProperties.hxx"

#include"TFEL/Math/Vector/VectorUtilities.hxx"
#include"TFEL/Math/Vector/VectorConcept.hxx"
#include"TFEL/Math/tvector.hxx"
#include"TFEL/Math/tmatrix.hxx"

namespace tfel
{
  
  namespace math
  {

    template<unsigned short N, unsigned short Mn,
	     unsigned short Mm,unsigned short In,
	     unsigned short Im,typename T>
    struct TVectorFromTinyMatrixColumnView2Expr
    {}; // end of struct TVectorFromTinyMatrixColumnView2Expr

    template<unsigned short N, unsigned short Mn,
	     unsigned short Mm,unsigned short In,
	     unsigned short Im,typename T>
    struct TvectorExpr<tvector<N,T>,TVectorFromTinyMatrixColumnView2Expr<N,Mn,Mm,In,Im,T> >
      : public TvectorConcept<TvectorExpr<tvector<N,T>,TVectorFromTinyMatrixColumnView2Expr<N,Mn,Mm,In,Im,T> > >,
	public tvector_base<TvectorExpr<tvector<N,T>,TVectorFromTinyMatrixColumnView2Expr<N,Mn,Mm,In,Im,T> >,N,T >
    {

      typedef EmptyRunTimeProperties RunTimeProperties;

      RunTimeProperties
      getRunTimeProperties() const
      {
	return RunTimeProperties();
      }

      TvectorExpr(tmatrix<Mn,Mm,T>& m_,
		  const unsigned short i_,
		  const unsigned short j_)
	: m(m_),
	  oi(i_),
	  oj(j_)
      {} // end of TvectorExpr

      const T&
      operator()(const unsigned short i) const
      {
	return this->m(In+(this->oi)*N+i,Im+this->oj);
      } // end of operator() const

      T&
      operator()(const unsigned short i)
      {
	return this->m(In+(this->oi)*N+i,Im+this->oj);
      } // end of operator()

      const T&
      operator[](const unsigned short i) const
      {
	return this->m(In+(this->oi)*N+i,Im+this->oj);
      } // end of operator[] const

      T&
      operator[](const unsigned short i)
      {
	return this->m(In+(this->oi)*N+i,Im+this->oj);
      } // end of operator[]

      using tvector_base<TvectorExpr>::operator =;
    
    protected:

      tmatrix<Mn,Mm,T>& m;
      const unsigned short oi;
      const unsigned short oj;
      
    private:
      
      //! simple checks
      TFEL_STATIC_ASSERT((N==1u)||(N==2u)||(N==3u));
      //! simple checks
      TFEL_STATIC_ASSERT((Im<Mm));
      //! simple checks
      TFEL_STATIC_ASSERT(N<=Mn-In));

    }; // end of struct TvectorExpr

    template<unsigned short N, unsigned short Mn,
	     unsigned short Mm,unsigned short In,
	     unsigned short Im,typename T = double>
    struct TVectorFromTinyMatrixColumnView2
    {
      typedef TvectorExpr<tvector<N,T>,TVectorFromTinyMatrixColumnView2Expr<N,Mn,Mm,In,Im,T> > type;
    }; // end of struct TVectorFromTinyMatrixColumnView2
    
  } // end of namespace math

} // end of namespace tfel

#endif /* LIB_TFEL_MATH_TVECTORFROMTINYMATRIXCOLUMNVIEW2_HXX */
